'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _classCallCheck = _interopDefault(require('@babel/runtime/helpers/classCallCheck'));
var _createClass = _interopDefault(require('@babel/runtime/helpers/createClass'));
var _toConsumableArray = _interopDefault(require('@babel/runtime/helpers/toConsumableArray'));
var _defineProperty = _interopDefault(require('@babel/runtime/helpers/defineProperty'));
var _typeof = _interopDefault(require('@babel/runtime/helpers/typeof'));
var deepmerge = require('deepmerge');
var fastDeepEqual = _interopDefault(require('fast-deep-equal'));
var fastMemoize = _interopDefault(require('fast-memoize'));
var nano = _interopDefault(require('nanoid'));
var objectOmit = _interopDefault(require('object.omit'));
var objectPick = _interopDefault(require('object.pick'));
var throttleDebounce = require('throttle-debounce');

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

/**
 * Type cast an argument. If no type is provided it will default to any.
 *
 * @param arg - the arg to typecast
 */
var Cast = function Cast(arg) {
  return arg;
};
/**
 * Shorthand for casting a value to it's boolean equivalent.
 *
 * @param value - the value to transform into a boolean
 *
 * @public
 */

var bool = function bool(value) {
  return !!value;
};
/**
 * A type name matcher for object types
 */

var TypeName;
/**
 * Alias of toString for non-dom environments
 */

(function (TypeName) {
  TypeName["Object"] = "Object";
  TypeName["RegExp"] = "RegExp";
  TypeName["Date"] = "Date";
  TypeName["Promise"] = "Promise";
  TypeName["Error"] = "Error";
  TypeName["Map"] = "Map";
  TypeName["Set"] = "Set";
})(TypeName || (TypeName = {}));

var toString = Object.prototype.toString;
/**
 * Retrieve the object type of a value via it's string reference. This is safer
 * than relying on instanceof checks which fail on cross-frame values.
 *
 * @param value - the object to inspect
 */

var getObjectType = function getObjectType(value) {
  var objectName = toString.call(value).slice(8, -1);
  return objectName;
};
/**
 * A helper for building type predicates
 *
 * @param type -  the name of the type to check for
 * @returns a predicate function for checking the value type
 */


var isOfType = function isOfType(type, test) {
  return function (value) {
    return _typeof(value) === type ? test ? test(value) : true : false;
  };
};
/**
 * Get the object type of passed in value. This avoids the reliance on
 * `instanceof` checks which are subject to cross frame issues as outlined in
 * this link https://bit.ly/1Qds27W
 *
 * @param type - the name of the object type to check for
 *
 * @public
 */


var isObjectOfType = function isObjectOfType(type) {
  return function (value) {
    return getObjectType(value) === type;
  };
};
/**
 * Check if an instance is the direct instance of the provided class.
 */


var isDirectInstanceOf = function isDirectInstanceOf(instance, Constructor) {
  return Object.getPrototypeOf(instance) === Constructor.prototype;
};
/**
 * Predicate check that value is undefined
 *
 * @param value - the value to check
 *
 * @public
 */

var isUndefined = isOfType('undefined');
/**
 * Predicate check that value is a string
 *
 * @param value - the value to check
 *
 * @public
 */

var isString = isOfType('string');
/**
 * Predicate check that value is a number.
 *
 * Also by default doesn't include NaN as a valid number.
 *
 * @param value - the value to check
 *
 * @public
 */

var isNumber = isOfType('number', function (val) {
  return !Number.isNaN(val);
});
/**
 * Predicate check that value is a function
 *
 * @param value - the value to check
 *
 * @public
 */

var isFunction = isOfType('function');
/**
 * Predicate check that value is null
 *
 * @param value - the value to check
 *
 * @public
 */

var isNull = function isNull(value) {
  return value === null;
};
/**
 * Predicate check that value is a class
 *
 * @deprecated Due to the current build process stripping out classes
 *
 * @param value - the value to check
 *
 * @public
 */

var isClass = function isClass(value) {
  return isFunction(value) && value.toString().startsWith('class ');
};
/**
 * Predicate check that value is boolean
 *
 * @param value - the value to check
 *
 * @public
 */

var isBoolean = function isBoolean(value) {
  return value === true || value === false;
};
/**
 * Predicate check that value is a symbol
 *
 * @param value - the value to check
 *
 * @public
 */

var isSymbol = isOfType('symbol');
/**
 * Helper function for Number.isInteger check allowing non numbers to be tested
 *
 * @param value - the value to check
 *
 * @public
 */

var isInteger = function isInteger(value) {
  return Number.isInteger(value);
};
/**
 * Helper function for Number.isSafeInteger allowing for unknown values to be
 * tested
 *
 * @param value - the value to check
 *
 * @public
 */

var isSafeInteger = function isSafeInteger(value) {
  return Number.isSafeInteger(value);
};
/**
 * Predicate check for whether passed in value is a plain object
 *
 * @param value - the value to check
 *
 * @public
 */

var isPlainObject = function isPlainObject(value) {
  if (getObjectType(value) !== TypeName.Object) {
    return false;
  }

  var prototype = Object.getPrototypeOf(value);
  return prototype === null || prototype === Object.getPrototypeOf({});
};
/**
 * Utility predicate check that value is either null or undefined
 *
 * @param value - the value to check
 *
 * @public
 */

var isNullOrUndefined = function isNullOrUndefined(value) {
  return isNull(value) || isUndefined(value);
};
/**
 * Predicate check that value is an object
 *
 * @param value - the value to check
 *
 * @public
 */

var isObject = function isObject(value) {
  return !isNullOrUndefined(value) && (isFunction(value) || isOfType('object')(value));
};
/**
 * A shorthand method for creating instance of checks.
 */

var isInstanceOf = function isInstanceOf(Constructor) {
  return function (value) {
    return isObject(value) && value instanceof Constructor;
  };
};
/**
 * Predicate check that value is a native promise
 *
 * @param value - the value to check
 *
 * @public
 */

var isNativePromise = function isNativePromise(value) {
  return isObjectOfType(TypeName.Promise)(value);
};
/**
 * Check to see if a value has the built in promise API.
 *
 * @param value - the value to check
 *
 * @public
 */

var hasPromiseAPI = function hasPromiseAPI(value) {
  return !isNull(value) && isObject(value) && isFunction(value.then) && isFunction(value.catch);
};
/**
 * Predicate check that value has the promise api implemented
 *
 * @param value - the value to check
 *
 * @public
 */


var isPromise = function isPromise(value) {
  return isNativePromise(value) || hasPromiseAPI(value);
};
/**
 * Predicate check that value is a RegExp
 *
 * @param value - the value to check
 *
 * @public
 */

var isRegExp = isObjectOfType(TypeName.RegExp);
/**
 * Predicate check that value is a date
 *
 * @param value - the value to check
 *
 * @public
 */

var isDate = isObjectOfType(TypeName.Date);
/**
 * Predicate check that value is an error
 *
 * @param value - the value to check
 *
 * @public
 */

var isError = isObjectOfType(TypeName.Error);
/**
 * Predicate check that value is a `Map`
 *
 * @param value - the value to check
 *
 * @public
 */

var isMap = function isMap(value) {
  return isObjectOfType(TypeName.Map)(value);
};
/**
 * Predicate check that value is a `Set`
 *
 * @param value - the value to check
 *
 * @public
 */

var isSet = function isSet(value) {
  return isObjectOfType(TypeName.Set)(value);
};
/**
 * Predicate check that value is an empty object
 *
 * @param value - the value to check
 *
 * @public
 */

var isEmptyObject = function isEmptyObject(value) {
  return isObject(value) && !isMap(value) && !isSet(value) && Object.keys(value).length === 0;
};
/**
 * Alias the isArray method.
 */

var isArray = Array.isArray;
/**
 * Predicate check that value is an empty array
 *
 * @param value - the value to check
 *
 * @public
 */

var isEmptyArray = function isEmptyArray(value) {
  return isArray(value) && value.length === 0;
};
/**
 * Capitalizes a string value.
 *
 * @param str - the string to capitalize.
 * @public
 */

var capitalize = function capitalize(str) {
  return str.charAt(0).toUpperCase() + str.slice(1);
};
/**
 * Removes leading and trailing whitespace from a string.
 *
 * @param str - the string to trim
 *
 * @public
 */

var trim = function trim(str) {
  return str.replace(/^ +| +$/g, '');
};
/**
 * Trim and conditionally capitalize string values.
 *
 * @param str - the string to format.
 *
 * @public
 */

var format = function format(str) {
  str = trim(str);
  return /^(?:webOS|i(?:OS|P))/.test(str) ? str : capitalize(str);
};
/**
 * Calls a function if defined and provides compile time type checking for the
 * passed in parameters.
 *
 * @param fn - the function to call if it exists
 * @param args - the rest of the parameters with types
 */

var callIfDefined = function callIfDefined(fn) {
  if (isFunction(fn)) {
    for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    fn.apply(void 0, args);
  }
};
/**
 * Finds all the regex matches for a string
 *
 * @param text - the text to check against
 * @param regexp - the regex (which should include a 'g' flag)
 *
 * @public
 */

var findMatches = function findMatches(text, regexp) {
  var results = [];
  var flags = regexp.flags;
  var match;

  if (!flags.includes('g')) {
    regexp = new RegExp(regexp.source, "g".concat(flags));
  }

  do {
    match = regexp.exec(text);

    if (match) {
      results.push(match);
    }
  } while (match);

  regexp.lastIndex = 0;
  return results;
};
/**
 * A utility function to clean up the Operating System name.
 *
 * @param os - the OS name to clean up.
 * @param pattern - a `RegExp` pattern matching the OS name.
 * @param label - a label for the OS.
 * @returns a cleaned up Operating System name
 *
 * @public
 */

var cleanupOS = function cleanupOS(os, pattern, label) {
  if (pattern && label) {
    os = os.replace(RegExp(pattern, 'i'), label);
  }

  var val = format(os.replace(/ ce$/i, ' CE').replace(/\bhpw/i, 'web').replace(/\bMacintosh\b/, 'Mac OS').replace(/_PowerPC\b/i, ' OS').replace(/\b(OS X) [^ \d]+/i, '$1').replace(/\bMac (OS X)\b/, '$1').replace(/\/(\d)/, ' $1').replace(/_/g, '.').replace(/(?: BePC|[ .]*fc[ \d.]+)$/i, '').replace(/\bx86\.64\b/gi, 'x86_64').replace(/\b(Windows Phone) OS\b/, '$1').replace(/\b(Chrome OS \w+) [\d.]+\b/, '$1').split(' on ')[0]);
  return val;
};
/**
 * A utility function to check whether the current browser is running on the
 * android platform.
 * @public
 */

var isAndroidOS = function isAndroidOS() {
  var ua = navigator.userAgent;
  var match = RegExp('\\b' + 'Android' + '(?:/[\\d.]+|[ \\w.]*)', 'i').exec(ua);

  if (!match) {
    return false;
  }

  return cleanupOS(match[0], 'Android', 'Android').includes('Android');
};
/**
 * Generate a random float between min and max. If only one parameter is
 * provided minimum is set to 0.
 *
 * @param min - the minimum value
 * @param max - the maximum value
 *
 * @public
 */

var randomFloat = function randomFloat(min, max) {
  if (!max) {
    max = min;
    min = 0.0;
  }

  return Math.random() * (max - min + 1) + min;
};
/**
 * Generate a random integer between min and max. If only one parameter is
 * provided minimum is set to 0.
 *
 * @param min - the minimum value
 * @param max - the maximum value
 *
 * @public
 */

var randomInt = function randomInt(min, max) {
  return Math.floor(randomFloat(min, max));
};
/**
 * Converts a string, including strings in camelCase or snake_case, into Start
 * Case (a variant of Title Case where all words start with a capital letter),
 * it keeps original single quote and hyphen in the word.
 *
 *   'management_companies' to 'Management Companies' 'managementCompanies' to
 *   'Management Companies' `hell's kitchen` to `Hell's Kitchen` `co-op` to
 *   `Co-op`
 *
 * @param str - the string to examine
 */

var startCase = function startCase(str) {
  return str.replace(/_/g, ' ').replace(/([a-z])([A-Z])/g, function (_, $1, $2) {
    return $1 + ' ' + $2;
  }).replace(/(\s|^)(\w)/g, function (_, $1, $2) {
    return $1 + $2.toUpperCase();
  });
};
var wordSeparators = /[\s\u2000-\u206F\u2E00-\u2E7F\\'!"#$%&()*+,\-./:;<=>?@[\]^_`{|}~]+/;
var capitals = /[A-Z\u00C0-\u00D6\u00D9-\u00DD]/g;
/**
 * Returns the kebab cased form of a string.
 *
 * Taken from
 * https://github.com/angus-c/just/blob/master/packages/string-kebab-case/index.js
 * kebabCase('the quick brown fox'); // 'the-quick-brown-fox'
 * kebabCase('the-quick-brown-fox'); // 'the-quick-brown-fox'
 * kebabCase('the_quick_brown_fox'); // 'the-quick-brown-fox'
 * kebabCase('theQuickBrownFox'); // 'the-quick-brown-fox'
 * kebabCase('theQuickBrown Fox'); // 'the-quick-brown-fox'
 * kebabCase('thequickbrownfox'); // 'thequickbrownfox' kebabCase('the - quick *
 * brown# fox'); // 'the-quick-brown-fox' kebabCase('theQUICKBrownFox'); //
 * 'the-q-u-i-c-k-brown-fox'
 */

var kebabCase = function kebabCase(str) {
  // replace capitals with space + lower case equivalent for later parsing
  return str.replace(capitals, function (match) {
    return ' ' + (match.toLowerCase() || match);
  }).trim().split(wordSeparators).join('-');
};
/**
 * This is required since it is not exported from `fastMemomize`
 *
 * @internalRemarks
 * TODO create a pull request to fix this.
 */

/**
 * Alias for caching function calls
 */
var memoize = fastMemoize;

/**
 * Generate a unique id
 *
 * @param params - the destructured params
 * @returns a unique string of specified length
 *
 * @public
 */
var uniqueId = function uniqueId() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
    prefix: ''
  },
      _ref$prefix = _ref.prefix,
      prefix = _ref$prefix === void 0 ? '' : _ref$prefix,
      size = _ref.size;

  return "".concat(prefix).concat(nano(size));
};
/**
 * Takes a number of elements from the provided array starting from the
 * zero-index
 *
 * @param arr - the array to take from
 * @param num - the number of items to take
 *
 * @public
 */

var take = function take(arr, num) {
  num = Math.max(Math.min(0, num), num);
  return arr.slice(0, num);
};
/**
 * Alias for picking properties from an object
 */

var pick = objectPick;
/**
 * Alias for excluding properties from an object
 */

var omit = objectOmit;
var omitUndefined = function omitUndefined(object) {
  return omit(object, function (value) {
    return !isUndefined(value);
  });
};
/**
 * Clones a plain object using object spread notation
 *
 * @param value - the value to check
 *
 * @public
 */

var clone = function clone(value) {
  if (!isPlainObject(value)) {
    throw new Error('An invalid value was passed into this clone utility. Expected a plain object');
  }

  return _objectSpread({}, value);
};
/**
 * Alias for fast deep equal
 */

var isEqual = fastDeepEqual;
/**
 * Create a unique array in a non-mutating manner
 *
 * @param array - the array which will be reduced to its unique elements
 * @param fromStart - when set to true the duplicates will be removed from the
 * beginning of the array. This defaults to false.
 * @return a new array containing only unique elements (by reference)
 *
 * @public
 */

var uniqueArray = function uniqueArray(array) {
  var fromStart = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var arr = fromStart ? _toConsumableArray(array).reverse() : array;
  var set = new Set(arr);
  return fromStart ? Array.from(set).reverse() : Array.from(set);
};
/**
 * Flattens an array
 *
 * @param array
 *
 * @public
 */

var flattenArray = function flattenArray(array) {
  return array.reduce(function (a, b) {
    return a.concat(Array.isArray(b) ? flattenArray(b) : b);
  }, []);
};
/**
 * Sometimes doing nothing is the best policy.
 */

var noop = function noop() {};
/**
 * Use this to completely overwrite an object when merging.
 *
 * ```ts
 * const source = { awesome: { a: 'a' } }
 * const target = { awesome: { b: 'b' } }
 * const result = deepMerge(source, target) // => { awesome: { a: 'a', b: 'b' } }
 *
 * const overwriteTarget = { awesome: Merge.overwrite({ b: 'b' }) }
 * const overwriteResult = deepMerge(source, overwriteTarget) // => { awesome: { b: 'b' } }
 * ```
 *
 */

var Merge =
/*#__PURE__*/
function () {
  _createClass(Merge, null, [{
    key: "overwrite",

    /**
     * Create an object that will completely replace the key when merging.
     *
     * @param [obj] - the object to replace the key with. When blank an empty
     * object is used.
     */
    value: function overwrite() {
      var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      return new Merge(obj);
    }
    /**
     * Sets the key to undefined thus fully deleting the key.
     */

  }, {
    key: "delete",
    value: function _delete() {
      return undefined;
    }
    /**
     * This can be used to mimic any object shape.
     */

  }]);

  function Merge() {
    var _this = this;

    var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, Merge);

    Object.keys(obj).forEach(function (key) {
      _this[key] = obj[key];
    });
  }

  return Merge;
}();
/**
 * A deep merge which only merges plain objects and Arrays. It clones the object
 * before the merge so will not mutate any of the passed in values.
 *
 * To completely remove a key you can use the `Merge` helper class which
 * replaces it's key with a completely new object
 */

var deepMerge = function deepMerge() {
  for (var _len2 = arguments.length, objects = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    objects[_key2] = arguments[_key2];
  }

  return deepmerge.all(objects, {
    isMergeableObject: isPlainObject
  });
};

/**
 * Clamps the value to the provided range.
 */
var clamp = function clamp(_ref2) {
  var min = _ref2.min,
      max = _ref2.max,
      value = _ref2.value;
  return value < min ? min : value > max ? max : value;
};
/**
 * Get the last element of the array.
 */

var last = function last(array) {
  return array[array.length - 1];
};
/**
 * Sorts an array while retaining the original order when the compare method
 * identifies the items as equal.
 *
 * `Array.prototype.sort()` is unstable and so values that are the same will
 * jump around in a non deterministic manner. Here I'm using the index as a
 * fallback. If two elements have the same priority the element with the lower
 * index is placed first hence retaining the original order.
 *
 * @param array - the array to sort
 * @param compareFn - compare the two value arguments `a` and `b` - return 0 for
 *                  equal - return number > 0 for a > b - return number < 0 for
 *                  b > a
 */

var sort = function sort(array, compareFn) {
  return _toConsumableArray(array).map(function (value, index) {
    return {
      value: value,
      index: index
    };
  }).sort(function (a, b) {
    return compareFn(a.value, b.value) || a.index - b.index;
  }).map(function (_ref3) {
    var value = _ref3.value;
    return value;
  });
};
/**
 * Get a property from an object or array by a string path or an array path.
 *
 * @param path - path to property
 * @param obj - object to retrieve property from
 */

var get = function get(path, obj, fallback) {
  if (!path || !path.length) {
    return isUndefined(obj) ? fallback : obj;
  }

  if (isString(path)) {
    path = path.split('.');
  }

  for (var ii = 0, len = path.length; ii < len && obj; ++ii) {
    if (!isPlainObject(obj) && !isArray(obj)) {
      return fallback;
    }

    obj = obj[path[ii]];
  }

  return isUndefined(obj) ? fallback : obj;
};
/**
 * Create a unique array of objects from a getter function or a property list.
 *
 * @param array - the array to extract unique values from
 * @param getValue - a getter function or a string with the path to the item
 * that is being used as a a test for uniqueness.
 * @param fromStart - when true will remove duplicates from the start rather
 * than from the end
 *
 * ```ts
 * import { uniqueBy } from '@remirror/core-helpers';
 *
 * const values = uniqueBy([{ id: 'a', value: 'Awesome' }, { id: 'a', value: 'ignored' }], item => item.id);
 * log(values) // => [{id: 'a', value: 'Awesome'}]
 *
 * const byKey = uniqueBy([{ id: 'a', value: 'Awesome' }, { id: 'a', value: 'ignored' }], 'id')
 * // Same as above
 * ```
 */

var uniqueBy = function uniqueBy(array, getValue) {
  var fromStart = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  var unique = [];
  var found = new Set();

  var makeFn = function makeFn(val) {
    return function (item) {
      return get(val, item);
    };
  };

  var getter = isFunction(getValue) ? getValue : makeFn(getValue);
  var arr = fromStart ? _toConsumableArray(array).reverse() : array;
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = arr[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _item = _step.value;

      var _value = getter(_item);

      if (!found.has(_value)) {
        found.add(_value);
        unique.push(_item);
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return fromStart ? unique.reverse() : unique;
};
/**
 * A typesafe implementation of `Object.entries`
 *
 * Taken from
 * https://github.com/biggyspender/ts-entries/blob/master/src/ts-entries.ts
 */

var entries = function entries(obj) {
  return Object.entries(obj);
};
/**
 * A typesafe implementation of `Object.keys`
 */

var keys = function keys(obj) {
  return Object.keys(obj);
};
/**
 * A more lenient typed version of `Array.prototype.includes` which allow less
 * specific types to be checked.
 */

var includes = function includes(array, item, fromIndex) {
  return array.includes(item, fromIndex);
};
/**
 * Create a range from start to end.
 *
 * If only start is provided it creates an array of the size provided. if start
 * and end are provided it creates an array who's first position is start and
 * final position is end. i.e. `length = (end - start) + 1`
 */

var range = function range(start, end) {
  return !isNumber(end) ? Array.from({
    length: Math.abs(start)
  }, function (_, index) {
    return (start < 0 ? -1 : 1) * index;
  }) : start <= end ? Array.from({
    length: end + 1 - start
  }, function (_, index) {
    return index + start;
  }) : Array.from({
    length: start + 1 - end
  }, function (_, index) {
    return -1 * index + start;
  });
};
/**
 * Check that a number is within the minimum and maximum bounds of a set of
 * numbers.
 *
 * @param value - the number to test
 */

var within = function within(value) {
  for (var _len3 = arguments.length, rest = new Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
    rest[_key3 - 1] = arguments[_key3];
  }

  var numbers = rest.filter(isNumber);
  return value >= Math.min.apply(Math, _toConsumableArray(numbers)) && value <= Math.max.apply(Math, _toConsumableArray(numbers));
};
/**
 * Safe implementation of hasOwnProperty with typechecking.
 *
 * @remarks
 *
 * See {@link https://eslint.org/docs/rules/no-prototype-builtins}
 *
 * @param obj - the object to check
 * @param key - the property to check
 *
 * @typeParam GObj - the object type
 * @typeParam GProperty - the property which can be a string | number | symbol
 */

var hasOwnProperty = function hasOwnProperty(obj, key) {
  return Object.prototype.hasOwnProperty.call(obj, key);
};

Object.keys(throttleDebounce).forEach(function (k) {
  if (k !== 'default') Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () {
      return throttleDebounce[k];
    }
  });
});
exports.Cast = Cast;
exports.Merge = Merge;
exports.bool = bool;
exports.callIfDefined = callIfDefined;
exports.capitalize = capitalize;
exports.clamp = clamp;
exports.cleanupOS = cleanupOS;
exports.clone = clone;
exports.deepMerge = deepMerge;
exports.entries = entries;
exports.findMatches = findMatches;
exports.flattenArray = flattenArray;
exports.format = format;
exports.get = get;
exports.hasOwnProperty = hasOwnProperty;
exports.includes = includes;
exports.isAndroidOS = isAndroidOS;
exports.isArray = isArray;
exports.isBoolean = isBoolean;
exports.isClass = isClass;
exports.isDate = isDate;
exports.isDirectInstanceOf = isDirectInstanceOf;
exports.isEmptyArray = isEmptyArray;
exports.isEmptyObject = isEmptyObject;
exports.isEqual = isEqual;
exports.isError = isError;
exports.isFunction = isFunction;
exports.isInstanceOf = isInstanceOf;
exports.isInteger = isInteger;
exports.isMap = isMap;
exports.isNativePromise = isNativePromise;
exports.isNull = isNull;
exports.isNullOrUndefined = isNullOrUndefined;
exports.isNumber = isNumber;
exports.isObject = isObject;
exports.isPlainObject = isPlainObject;
exports.isPromise = isPromise;
exports.isRegExp = isRegExp;
exports.isSafeInteger = isSafeInteger;
exports.isSet = isSet;
exports.isString = isString;
exports.isSymbol = isSymbol;
exports.isUndefined = isUndefined;
exports.kebabCase = kebabCase;
exports.keys = keys;
exports.last = last;
exports.memoize = memoize;
exports.noop = noop;
exports.omit = omit;
exports.omitUndefined = omitUndefined;
exports.pick = pick;
exports.randomFloat = randomFloat;
exports.randomInt = randomInt;
exports.range = range;
exports.sort = sort;
exports.startCase = startCase;
exports.take = take;
exports.trim = trim;
exports.uniqueArray = uniqueArray;
exports.uniqueBy = uniqueBy;
exports.uniqueId = uniqueId;
exports.within = within;
//# sourceMappingURL=core-helpers.cjs.js.map
